function dxdt = twoTanks(t,x,u,p)
%
% dxdt: Rate of change state vars
%
% x:    State variables
% u:    Control vars
% p:    Parameters
% t:    Time
%
% Controlled two-tank systems
%
% Using process notation
% dV_1(t)/dt = q_1(t) - q_2(t)
% dV_2(t)/dt = q_2(t) -k/B * V_2(t)
%
% Using control notation
%
% dx_1(t)/dt = u_1(t) - u_2(t)
% dx_2(t)/dt = u_2(t) - p_1/p_2 * x_2(t)

%% From control- to process-notation

% State variables 
V(1) = x(1);
V(2) = x(2);

% Control variables
q(1) = u(1);
q(2) = u(2);

% Parameters
k = p(1);
B = p(2);

%% Dynamic model of the system
dVdt = [  
        q(1) - q(2); ...        % Rate of change of V_1, state var x_1
                                % -> dx_1/dt = f_1(x, u; p)
        q(2) - k/B * V(2)       % Rate of change of V_2, state var x_2
                                % -> dx_2/dt = f_2(x, u; p)            
       ];
   
%% From process- to control-notation
dxdt = dVdt;

end

