clear all; close all; clc

%% Simulation of a dynamic process, the two-tank system, using explicit 
%  Euler integration 
% 
%  In this exercise we are going to simulate the time-evolution of the two- 
%  tank, starting from some given initial conditions of the state variables
%
%  dV_1(t)/dt = q_1(t) - q_2(t)
%  dV_2(t)/dt = q_2(t) - q_3(t)
%  
%  with q_3(t) = k/B * V_2(t)
%
%  We simulate the process model when subjected to 3 different scenarios
%
%  Case 0. Initial state non-zero, A constant zero controls, 
%           -> Initially non-empty tanks, V_1(0)!=0, V_2(0)!=0
%           -> Fixed at zero flow rates q_1(t) and q_2(t), 
%              q_1(t) = 0 and q_2(t) = 0
%
%  Case 1. Initial state zero, whatever constant controls,
%           -> Initially empty tanks, V_1(0)=0, V_2(0)=0
%           -> Fixed flow rates q_1(t) and q_2(t0, 
%              q_1(t) = const and q_2(t) = const
%
%  Case 2. Time-varying control signal, initial state whatever
%           -> Whatever initial volume in the tanks, V_1(0) and V_2(0)
%           -> Whatever dynamic flow-rates q_1(t) and q_2(t)

%% Define the parameters of the system
    B = 1.0; % Cross-section are of the tank (say, we use m^2)
    k = 0.9; % Coefficient   (adim)

    p = [k; B]; % Plug them into a 2x1 vector of parameters

%% Define the simulation time (t0 to tF, in deltaTime long intervals)
    tF = 10;                        % Final time of the simulation (min)
    deltaTime = 1/60;               % Step size (min)
    
    nSteps = tF/deltaTime;          % Number of time steps between T0 and Tf (adim)
    nSteps = round(nSteps);         % Round to closest integer 

%% Pick a case to simulate
caseN = input('Type in case number (0, 1, or 2): '); 

switch caseN    
    case 0
    %% Case 0. 
    
    % Set the initial conditions, initial values of the state variables
    % (The volumes at t0, collectively call them X0)
    V_1_at_0 = 1;                   % Initial condition for state variable 1 
    V_2_at_0 = 1;                   % Initial condition for state variable 2

    X0 = [V_1_at_0; ...
          V_2_at_0];                % Into a 2 x 1 vector (control notation)
    
    % Set the evolution of the input signals q_1(0->tF) and q_2(0-tF)
    % between t0 and tF (collectively call them U)
    q_1 = 0 * ones(1,nSteps);       % nSteps values of control variable 1
    q_2 = 0 * ones(1,nSteps);       % nSteps values of control variable 2

    U = [q_1; ...
         q_2];                      % Into a 2 x nSteps matrix (control notation)
     
    aU = [q_1; ...                  % Actual control that can be applied
         q_2];                      % Into a 2xN matrix (control notation)
 
    case 1
    %% Case 1.  
    
    % Set X0
    V_1_at_0 = 0;                   % Initial condition for state variable 1 
    V_2_at_0 = 0;                   % Initial condition for state variable 2
    
    X0 = [V_1_at_0; ...
          V_2_at_0];                % Into a 2 x 1 vector (control notation)
      
    % Set q_1(0->tF) and q_2(0->tF)
  
    q_1 = 1.0 * ones(1,nSteps);       % nSteps values of control variable 1
    q_2 = 0.1 * ones(1,nSteps);       % nSteps values of control variable 2

    U = [q_1; q_2];                 % Into a 2 x nSteps matrix (control notation)
     
    aU = [q_1; ...                  % Actual control that can be applied
         q_2];                      % Into a 2xN matrix (control notation)

    case 2
    %% Case 2. 
    
    % Set X0
    V_1_at_0 = 1;                   % Initial condition for state variable 1 
    V_2_at_0 = 1;                   % Initial condition for state variable 2

    X0 = [V_1_at_0; V_2_at_0];      % Into a 2x1 vector (control notation)
    
    % Set U

    q_1 = 1 * ones(1,nSteps);       % N values of control variable 1
    q_2 = 1 * ones(1,nSteps);       % N values of control variable 2
    
        q_1(1:floor(nSteps/4)) = 1;     
        q_1(floor(1*nSteps/4)+1:floor(2*nSteps/4)) = 2;
        q_1(floor(2*nSteps/4)+1:floor(3*nSteps/4)) = 1;
        q_1(floor(3*nSteps/4)+1:nSteps) = 3;
        
        q_2(1:floor(nSteps/4)) = 2;     
        q_2(floor(1*nSteps/4)+1:floor(2*nSteps/4)) = 0;
        q_2(floor(2*nSteps/4)+1:floor(3*nSteps/4)) = 2;
        q_2(floor(3*nSteps/4)+1:nSteps) = 1;

    U = [q_1; ...
         q_2];                      % Into a 2xN matrix (control notation)
     
    aU = [q_1; ...                  % Actual control that can be applied
         q_2];                      % Into a 2xN matrix (control notation)
end

%% Simulation the time-evolution of the process, under the selected case

% Initialise the solution, to be replaced during simulation
    V_1 = zeros(1,nSteps);          % nSteps values of state variable 1
    V_1(1) = V_1_at_0;              % Assign initial value, known

    V_2 = zeros(1,nSteps);          % nSteps values of state variable 2
    V_2(1) = V_2_at_0;              % Assign initial values, known  

    X = [V_1;V_2];                  % Into a 2 x nSteps matrix (control notation)

% Initialise the measurements, to be replaces during simulation 
    Y = V_2/B - V_1/B;

% Simulate the time evolution of the two-tank system
%      fprintf('\n== Simulation ==\nitime | time (min) |  V_1  |  V_2  |  q_1  |  q_2  | dV_1/dt | dV_2/dt \n')
    for itime = 2:nSteps
        
        % Sanity checks to remove negative volumes in the tanks
        for ix = 1:numel(X0) 
            if X(ix,itime-1) < 0 
               X(ix,itime-1) = 0; 
            end
        end 
        
        % Sanity check to limit q_2 x deltaTime to be at most what's inside  tank, V_1
        if U(2,itime-1) >= X(1,itime-1)/deltaTime
           aU(2,itime-1) = X(1,itime-1)/deltaTime;
        end    
        
        % Compute new value of the state variables (dynamics)

        X(:,itime) = X(:,itime-1) + deltaTime * twoTanks(1,X(:,itime-1),aU(:,itime-1),p);
        
%          fprintf('%3d   | %-10.2f | %-3.3f | %-3.3f | %-3.3f | %-3.3f | %-7.2f | %-7.2f \n', ...
%           itime-1, (itime-1)*deltaTime, X(1,itime-1), X(2,itime-1), U(1,itime-1), U(2,itime-1), ...
%           twoTanks(1,X(:,itime-1),aU(:,itime-1),p)')
%          pause(0.01)
        
        % Compute the new value of the output variable (read-out)
        Y(:,itime) = X(2,itime)/B - X(1,itime)/B;
    end
    
% Plot simulation results  
    plotTwotanks(U,aU,X,Y,p,nSteps,deltaTime)











